#pragma once

#include "Slate.h"
#include "SkillButtonStyle.h"
#include "GUIStyle.h"

/** SSkillButton is a button that most likely will be put inside SSkillsPanel.
*	Default style should be placed under Content/UI and named SkillButton_Default.*/
class GUILIBRARY_API SSkillButton : public SCompoundWidget
{
public:
	SLATE_BEGIN_ARGS(SSkillButton)
		: _SkillButtonStyle(&FGUIStyle::Get().GetWidgetStyle<FSkillButtonWidgetStyle>("SkillButton_Default"))
		, _DrawHotkeyText(true)
		, _HotkeyTextInside(false)
		, _ShowHotkeyTextOnHover(false)
	{}
		/* Style for this button. */
		SLATE_STYLE_ARGUMENT(FSkillButtonWidgetStyle, SkillButtonStyle)

		/* Hotkey text that will be displayed below or inside button. */
		SLATE_ATTRIBUTE(FText, HotkeyText)

		/* Command list is required to execute mapped commands. */
		SLATE_ARGUMENT(TSharedPtr<FUICommandList>, CommandList)

		/* Command that will be executed if set. */
		SLATE_ARGUMENT(TSharedPtr<FUICommandInfo>, CommandInfo)

		/* Should this button draw Hotkey text. */
		SLATE_ARGUMENT(bool, DrawHotkeyText)

		/* Should this button draw Hotkey text inside of it. */
		SLATE_ARGUMENT(bool, HotkeyTextInside)

		/* Should hotkey text be displayed only on mouse hover. */
		SLATE_ARGUMENT(bool, ShowHotkeyTextOnHover)

		/* Action used to execute bound action and determine if button should be disabled (using ButtonAction.CanExecute). */
		SLATE_ARGUMENT(FUIAction, ButtonAction)

		/* Simple delegate that will be executed on click. */
		SLATE_EVENT(FSimpleDelegate, OnButtonClick)
	SLATE_END_ARGS()

	void Construct(const FArguments& InArgs);

	/*
	*	Toggle visibility of hotkey text.
	*	@param bShow true == show; false == hide
	*/
	void ShowhotkeyText(bool bShow);

protected:

	/*
	*	Use ButtonAction to see if button should be enabled.
	*/
	bool IsButtonEnabled() const;

	/*
	*	Return Hotkeytext widget visibility.
	*/
	EVisibility GetHotkeyTextVisibility() const;

	/*
	*	Return input gesture as text. If commandInfo is set it will use CommandInfo->GetInputText(), HotkeyText otherwise.
	*/
	FText GetInputGesture() const;

	/*
	*	Do the magic.
	*/
	FReply OnClick();



	/* Widget containing Hotkey text.*/
	TSharedPtr<SWidget> HotkeyTextWidget;

	/* Command list is required to execute mapped commands. */
	TSharedPtr<FUICommandList> CommandList;

	/* Command that will be executed on click if set. */
	TSharedPtr<FUICommandInfo> CommandInfo;

	/* Actual hotkey text. */
	TAttribute<FText> HotkeyText;

	/* Button action. */
	FUIAction ButtonAction;

	/* Simple callback for button click. */
	FSimpleDelegate OnButtonClick;
};